/**
  * formily v1.0.0-alpha.13
  *
  * @link https://vue-formily.netlify.app
  * @source https://github.com/vue-formily/formily
  * (c) 2021 An Ha
  * @license MIT
  */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.VueFormily = {}));
}(this, (function (exports) { 'use strict';

  const _toString = Object.prototype.toString;
  /**
   * Strict object type check. Only returns true
   * for plain JavaScript objects.
   */

  function isPlainObject(obj) {
    return _toString.call(obj) === '[object Object]';
  }
  function isNumber(value) {
    return typeof value === 'number';
  }
  function isString(value) {
    return typeof value === 'string';
  }
  /**
   * Check if the input string contains only numeric characters.
   */

  function isNumeric(value) {
    const val = String(value);
    return val.length > 0 && !isNaN(+val);
  }
  function isFunction(fn) {
    return typeof fn === 'function';
  }

  /**
   * Merge objects or arrays
   */

  function merge(target, ...sources) {
    return sources.reduce((acc, source) => {
      const keys = Object.keys(source || []);
      keys.forEach(key => {
        const item = source[key];

        if (isPlainObject(item)) {
          acc[key] = merge({}, acc[key], item);
        } else if (Array.isArray(item)) {
          acc[key] = merge([], acc[key], item);
        } else {
          const descriptor = Object.getOwnPropertyDescriptor(source, key);
          Object.defineProperty(acc, key, descriptor || item);
        }
      });
      return acc;
    }, target);
  }
  const rkey = /([^[\]]+)/g;
  /**
   * Find the first found value in objects or arrays by its path
   */

  function get(path, ...args) {
    const length1 = args.length;
    const paths = isString(path) ? path.split('.') : path;
    const length2 = paths.length;

    for (let i = 0; i < length1; i++) {
      let value = args[i];
      let j = -1;

      while (value && ++j < length2) {
        const fullName = paths[j];
        const m = fullName.match(rkey);

        if (m) {
          const length3 = m.length;

          for (let k = 0; k < length3; k++) {
            const name = m[k];

            if ((Array.isArray(value) || isPlainObject(value)) && name in value) {
              value = value[name];

              if (j === length2 - 1 && k === length3 - 1) {
                return value;
              }
            }
          }
        }
      }
    }

    return undefined;
  }
  /**
   * Returns the index of the first element in the array that satisfies the provided testing function.
   */

  function findIndex(arr = [], fn) {
    const length = arr.length;

    for (let i = 0; i < length; i++) {
      if (fn(arr[i], i, arr)) {
        return i;
      }
    }

    return -1;
  }

  function normalizeRules(rules = [], type = null) {
      const _rules = [];
      rules.forEach((rule) => {
          /**
           * Only apply validation rule to 'function' or 'undefined' or field that has type is included in 'types' property of the rule
           */
          if (isFunction(rule) || !rule.for || (type && rule.for.includes(type))) {
              _rules.push(rule);
          }
      });
      return _rules;
  }
  function invalidateSchemaValidation(sv, reason, infos) {
      sv.valid = false;
      sv.reason = reason;
      sv.infos = infos;
  }
  function getSchemaAcceptance(schema, type) {
      const sv = { valid: false };
      const i = {
          accepted: false,
          sv
      };
      if ('_is' in schema) {
          i.accepted = true;
          if (schema._is === type) {
              sv.valid = true;
          }
      }
      else {
          sv.valid = true;
      }
      return i;
  }
  function acceptSchema(schema, type) {
      schema._is = type;
  }

  function isUndefined(value) {
      return value === undefined;
  }

  function logMessage(message, infos) {
      const infosText = infos
          ? `(${Object.keys(infos)
            .map((name) => `${name}: "${infos[name]}"`)
            .join(', ')}) `
          : '';
      return `[vue-formily] ${infosText}${message}`;
  }
  function toString(value) {
      return '' + value;
  }

  function readonlyDumpProp(obj, key, value, descriptor) {
      return def(obj, key, Object.assign({ value, writable: false, configurable: false }, descriptor));
  }
  function dumpProp(obj, key, value, descriptor) {
      return def(obj, key, Object.assign({ value, writable: true, configurable: false }, descriptor));
  }
  function def(obj, key, descriptor) {
      return Object.defineProperty(obj, key, Object.assign({ configurable: true, enumerable: true }, descriptor));
  }
  function readonlyDef(obj, key, get, descriptor) {
      return def(obj, key, Object.assign({ get }, descriptor));
  }

  function genFields(fields, parent, ...args) {
      const elements = parent._config.elements;
      const length = elements.length;
      let invalidSchema;
      if (!length) {
          throw new Error(logMessage('No form elements have been registed yet'));
      }
      return fields.map(schema => {
          for (let i = 0; i < length; i++) {
              const F = elements[i];
              const accepted = F.accept(schema);
              if (accepted.valid) {
                  return F.create(schema, parent, ...args);
              }
              invalidSchema = schema;
          }
          throw new Error(logMessage(`Failed to create form elmenent, caused by schema:\n ${JSON.stringify(invalidSchema, null, 2).slice(0, 50)}\n\t...\n`));
      });
  }
  function cascadeRules(parentRules, fields) {
      return parentRules
          ? fields.map(fieldSchema => {
              const { rules = [] } = fieldSchema;
              parentRules.forEach(parentRule => {
                  const index = findIndex(rules, (rule) => rule.name === parentRule.name);
                  const rule = rules[index];
                  if (!isFunction(parentRule) &&
                      parentRule.cascade &&
                      (!rule || rule.inherit !== false)) {
                      rules[index < 0 ? 0 : index] = merge({}, parentRule, rule);
                  }
              });
              if (rules.length) {
                  fieldSchema.rules = rules;
              }
              return fieldSchema;
          })
          : fields;
  }
  function genHtmlName(Element, ancestors) {
      const keysPath = ancestors
          ? ancestors.reduce((acc, fe) => {
              return 'index' in fe ? [...acc, '' + fe.index] : [...acc, fe.formId];
          }, [])
          : [];
      const [root, ...rest] = [...keysPath, 'index' in Element ? '' + Element.index : Element.formId];
      const htmlName = rest ? `${root}[${rest.join('][')}]` : root;
      return Element.type === 'set' ? `${htmlName}[]` : htmlName;
  }
  function getProp(element, path, options = {}) {
      let prop = get(path, element.props);
      const parent = element.parent;
      if (options.up && isUndefined(prop) && parent) {
          prop = getProp(parent, path, options);
      }
      return prop;
  }
  function genProps(source, properties, ...args) {
      for (const key in properties) {
          const prop = properties[key];
          const newSource = isPlainObject(prop) ? {} : Array.isArray(prop) ? [] : null;
          if (newSource) {
              source[key] = genProps.call(this, newSource, prop, ...args);
          }
          else if (isFunction(prop)) {
              def(source, key, {
                  get: () => prop.call(this, this, ...args)
              });
          }
          else {
              const translater = this.plugs.i18n;
              source[key] = isString(prop) && translater ? translater.translate(prop, this, ...args) : prop;
          }
      }
      return source;
  }

  function emit(name, ...args) {
      const event = this.events[name];
      if (event) {
          const handlers = event.handlers;
          handlers.forEach(h => h.call(this, ...args));
      }
  }
  function off(name, handler) {
      const event = this.events[name];
      if (event) {
          const { handlers, options = {} } = event;
          if (!options.noOff) {
              if (handler) {
                  handlers.splice(handlers.indexOf(handler), 1);
              }
              else {
                  delete this.events[name];
              }
          }
      }
  }
  function handleEventHandlers(eventName, fn) {
      Object.keys(this.events).forEach((name) => {
          const [n] = parseName(name);
          if (n === eventName) {
              fn(name);
          }
      });
  }
  function parseName(name) {
      return name.split(':');
  }
  class Evento {
      constructor() {
          dumpProp(this, 'events', {});
      }
      emit(name, ...args) {
          const [eventName, namespace] = parseName(name);
          if (namespace) {
              emit.call(this, name, ...args);
          }
          else {
              handleEventHandlers.call(this, eventName, (n) => emit.call(this, n, ...args));
          }
          return this;
      }
      once(name, handler) {
          this.on(name, (...args) => {
              this.off(name, handler);
              handler.call(this, ...args);
          });
          return this;
      }
      on(name, handler, options) {
          const event = this.events[name] || { handlers: [] };
          const handlers = event.handlers;
          if (!handlers.includes(handler)) {
              handlers.push(handler);
          }
          if (options) {
              event.options = options;
          }
          this.events[name] = event;
          return this;
      }
      off(name, handler) {
          const [eventName, namespace] = parseName(name);
          if (namespace) {
              off.call(this, name, handler);
          }
          else {
              handleEventHandlers.call(this, eventName, (n) => off.call(this, n, handler));
          }
          return this;
      }
  }

  class Objeto extends Evento {
      constructor() {
          super(...arguments);
          this._d = {};
      }
  }

  function genElementAncestors(elem) {
      const path = [];
      let parent = elem.parent;
      while (parent) {
          path.unshift(parent);
          parent = parent.parent;
      }
      return path.length ? path : null;
  }
  let _options = {
      silent: true
  };
  class Element extends Objeto {
      constructor(schema, parent) {
          super();
          this.props = {};
          this.data = new WeakMap();
          this.shaked = false;
          this.parent = parent || null;
          const data = this._d;
          readonlyDumpProp(data, 'schema', schema);
          const { model, props = {}, on = {}, options } = schema;
          readonlyDumpProp(this, 'model', model || this.formId);
          dumpProp(data, 'ancestors', genElementAncestors(this));
          dumpProp(data, 'options', merge({}, _options, options));
          this.addProps(props);
          Object.keys(on).map(name => this.on(name, on[name]));
      }
      static register(options) {
          _options = merge(_options, options);
      }
      get options() {
          return this._d.options;
      }
      get validation() {
          return this._d.validation;
      }
      get error() {
          if (!this.shaked || this.valid) {
              return null;
          }
          return this.validation.errors ? this.validation.errors[0] : null;
      }
      getProps(path, options) {
          return getProp(this, path, options);
      }
      addProps(props, ...args) {
          genProps.call(this, this.props, props, ...args);
      }
      get formId() {
          return this._d.schema.formId || '' + Date.now();
      }
      get htmlName() {
          return this.getHtmlName();
      }
      get valid() {
          return this.isValid();
      }
      getHtmlName() {
          return genHtmlName(this, this._d.ancestors);
      }
      shake() {
          this.shaked = true;
      }
      cleanUp() {
          this.shaked = false;
      }
  }

  class Rule extends Objeto {
      constructor(rule) {
          super();
          readonlyDumpProp(this, 'name', rule.name || Date.now());
          dumpProp(this, 'message', null);
          const data = this._d;
          data.error = null;
          data.valid = true;
          if (isFunction(rule)) {
              this.validator = rule;
          }
          else if (isPlainObject(rule)) {
              this.validator = rule.validator;
              this.setMessage(rule.message);
          }
      }
      get valid() {
          return this._d.valid;
      }
      get error() {
          return this._d.error;
      }
      setMessage(message = null) {
          this.message = message;
      }
      reset() {
          this._d.error = null;
          this._d.valid = true;
      }
      async validate(value, props = {}, ...args) {
          const { _d: data, plugs = {} } = this;
          const translater = plugs.i18n;
          let error = null;
          let valid = true;
          let result = true;
          if (isFunction(this.validator)) {
              result = await this.validator(value, props, ...args);
          }
          this.emit('validate', this);
          if (result === false || isString(result)) {
              error = result || this.message;
              valid = false;
          }
          data.error = translater && isString(error) ? translater.translate(error, ...args) : error;
          data.valid = valid;
          this.emit('validated', this);
          return this;
      }
  }

  class Validation extends Objeto {
      constructor(rules) {
          super();
          this.rules = [];
          if (rules) {
              this.addRules(rules);
          }
      }
      get valid() {
          return !this.rules.some(rule => !rule.valid);
      }
      get errors() {
          if (this.valid) {
              return null;
          }
          const errors = this.rules.map(rule => rule.error).filter(error => error);
          return errors.length ? errors : null;
      }
      addRules(rulesOrSchemas, { from } = {}) {
          from = isNumber(from) ? from++ : -Infinity;
          return rulesOrSchemas.map((schema) => {
              return this.addRule(schema, { from: from++ });
          });
      }
      removeRules(removes) {
          return removes.map(remove => this.removeRule(remove));
      }
      addRule(ruleOrSchema, { from } = {}) {
          const rule = new Rule(ruleOrSchema);
          const currentRule = this[rule.name];
          if (currentRule) {
              this.removeRule(currentRule);
          }
          const length = this.rules.length;
          this.rules.splice(isNumber(from) && from >= 0 && from <= length ? from : length, 0, rule);
          this[rule.name] = rule;
          return rule;
      }
      removeRule(remove) {
          const index = findIndex(this.rules, ({ name }) => {
              const n = remove instanceof Rule ? remove.name : remove;
              return name === n;
          });
          const [removed] = index !== -1 ? this.rules.splice(index, 1) : [];
          delete this[removed.name];
          return removed;
      }
      reset() {
          this.rules.forEach(rule => rule.reset());
      }
      async validate(value, options = {}, ...args) {
          const { excluded, get } = options;
          this.emit('validate', this);
          if (this.rules) {
              let rules = get ? this.rules.filter(({ name }) => get.includes(name)) : this.rules;
              rules = excluded ? rules.filter(({ name }) => !excluded.includes(name)) : rules;
              await Promise.all(rules.map(async (rule) => await rule.validate(value, ...args)));
          }
          this.emit('validated', this);
          return this;
      }
  }

  const FORM_TYPE = 'group';
  async function onFieldChanged(...args) {
      const [field] = args;
      const { valid, model } = field;
      this._d.pending = true;
      if (valid) {
          let value = this._d.value;
          if (!value) {
              value = this._d.value = {};
          }
          readonlyDef(value, model, () => this[model].value);
      }
      if (this.options.silent) {
          await this.validate({ cascade: false });
      }
      this.emit('changed', this, ...args);
  }
  class Group extends Element {
      constructor(schema, parent) {
          super(schema, parent);
          const accepted = Group.accept(schema);
          if (!accepted.valid) {
              throw new Error(logMessage(`invalid schema, ${accepted.reason}`, accepted.infos));
          }
          readonlyDumpProp(this, 'formType', FORM_TYPE);
          readonlyDumpProp(this, 'type', 'enum');
          if (schema.rules) {
              schema.fields = cascadeRules(schema.rules, schema.fields);
          }
          this.fields = genFields(schema.fields, this);
          this.fields.forEach(field => {
              this[field.model] = field;
              field.on('changed:formy', (...args) => onFieldChanged.apply(this, args), { noOff: true });
          });
          this._d.validation = new Validation(normalizeRules(schema.rules, this.type));
          this._d.value = null;
      }
      static accept(schema) {
          const { accepted, sv } = getSchemaAcceptance(schema, FORM_TYPE);
          if (!accepted) {
              const { fields, formId } = schema;
              if (!Array.isArray(fields)) {
                  invalidateSchemaValidation(sv, '`fields` must be an array.', { formId });
              }
              if (sv.valid) {
                  acceptSchema(schema, FORM_TYPE);
              }
          }
          return sv;
      }
      static create(schema, parent) {
          return new Group(schema, parent);
      }
      get pending() {
          return this._d.pending;
      }
      get value() {
          return this._d.value;
      }
      async setValue(obj) {
          if (!isPlainObject(obj)) {
              throw new Error(logMessage('Invalid value, Group value must be an object'));
          }
          await Promise.all(Object.keys(obj).map(async (model) => {
              const field = this[model];
              if (field) {
                  await field.setValue(obj[model]);
              }
          }));
          return this.value;
      }
      shake({ cascade = true } = {}) {
          super.shake();
          if (cascade) {
              this.fields.forEach(field => field.shake());
          }
      }
      isValid() {
          return this.validation.valid && !this.fields.some(field => !field.valid);
      }
      reset() {
          this.cleanUp();
          this.fields.forEach((field) => field.reset());
          this.validation.reset();
      }
      async clear() {
          this.cleanUp();
          await Promise.all(this.fields.map(async (field) => await field.clear()));
      }
      async validate({ cascade = true } = {}) {
          this.emit('validate', this);
          if (cascade) {
              await Promise.all(this.fields.filter((field) => 'validate' in field).map(async (field) => await field.validate()));
          }
          await this.validation.validate(this.value, {}, this.props, this);
          if (!this.valid) {
              this._d.value = null;
          }
          this._d.pending = false;
          this.emit('validated', this);
      }
  }
  Group.FORM_TYPE = FORM_TYPE;

  class Form extends Group {
  }

  const FORM_TYPE$1 = 'field';
  const casts = {
      string(value) {
          return value !== null ? toString(value) : null;
      },
      number(value) {
          return isNumeric(value) ? +value : null;
      },
      boolean(value) {
          return !!(value === true || value === 'true');
      },
      date(value) {
          const date = new Date(value);
          return !isNaN(date.getTime()) ? date : null;
      }
  };
  function formatter() {
      const { type, _d: data, plugs = {} } = this;
      const { format } = data.schema;
      const FORMATER = `${type}Format`;
      const formatter = plugs[FORMATER];
      const translater = plugs.i18n;
      let result = null;
      if (format && formatter) {
          const formatting = isFunction(format) ? format.call(this, this) : format;
          const translated = translater ? translater.translate(formatting, this) : formatting;
          result = formatter.format(translated, this);
      }
      return result;
  }
  class Field extends Element {
      constructor(schema, parent) {
          super(schema, parent);
          const accepted = Field.accept(schema);
          if (!accepted.valid) {
              throw new Error(logMessage(`[Schema error] ${accepted.reason}`, accepted.infos));
          }
          const { type, rules, default: defu } = schema;
          readonlyDumpProp(this, 'formType', FORM_TYPE$1);
          readonlyDumpProp(this, 'type', type || 'string');
          this._d.validation = new Validation(normalizeRules(rules, this.type));
          const hasDefault = !isUndefined(defu);
          readonlyDumpProp(this, 'default', hasDefault ? defu : null);
          const value = !isUndefined(schema.value) ? schema.value : defu;
          const data = this._d;
          data.typed = null;
          data.formatted = null;
          data.pending = false;
          this.setCheckedValue(schema.checkedValue);
          this.setValue(value);
      }
      static accept(schema) {
          const { type: schemaType, formId } = schema;
          const type = schemaType ? Field[`FIELD_TYPE_${schemaType.toUpperCase()}`] : 'string';
          const { accepted, sv } = getSchemaAcceptance(schema, type);
          if (!accepted) {
              if (!type) {
                  invalidateSchemaValidation(sv, 'Invalid `type`', { formId });
              }
              if (sv.valid) {
                  acceptSchema(schema, type);
              }
          }
          return sv;
      }
      static create(schema, parent) {
          return new Field(schema, parent);
      }
      get pending() {
          return this._d.pending;
      }
      get formatted() {
          return this._d.formatted;
      }
      get raw() {
          return this._d.raw;
      }
      set raw(value) {
          this.setRaw(value);
      }
      async setRaw(value) {
          await this.setValue(value);
      }
      get value() {
          return this._d.typed;
      }
      set value(value) {
          this.setValue(value);
      }
      async setValue(value) {
          const _d = this._d;
          const curRaw = _d.raw || '';
          _d.raw = !isUndefined(value) ? toString(value) : curRaw;
          const raw = _d.raw;
          if (this.options.silent) {
              await this.validate();
          }
          if (raw !== curRaw) {
              this.emit('changed', this, curRaw, raw);
          }
          return this.value;
      }
      setCheckedValue(checkedValue) {
          this._d.checkedValue = checkedValue;
      }
      get checked() {
          const { checkedValue } = this._d;
          if (this.type === Field.FIELD_TYPE_BOOLEAN) {
              return this.value;
          }
          return !isUndefined(checkedValue) && toString(this.value) === checkedValue;
      }
      isValid() {
          return this.validation.valid;
      }
      reset() {
          this._d.raw = this.default !== null ? this.default : '';
          this.cleanUp();
          this.validation.reset();
      }
      async clear() {
          this.cleanUp();
          await this.setValue('');
      }
      async validate() {
          const raw = this.raw;
          const cast = casts[this.type];
          const data = this._d;
          data.pending = true;
          this.emit('validate', this);
          const typed = cast(raw);
          const { valid } = await this.validation.validate(typed, {}, this.props, this);
          data.typed = typed !== null && valid ? typed : null;
          data.formatted = formatter.call(this);
          data.pending = false;
          this.emit('validated', this);
      }
  }
  Field.FORM_TYPE = FORM_TYPE$1;
  Field.FIELD_TYPE_STRING = 'string';
  Field.FIELD_TYPE_NUMBER = 'number';
  Field.FIELD_TYPE_BOOLEAN = 'boolean';
  Field.FIELD_TYPE_DATE = 'date';

  const FORM_TYPE$2 = 'collection';
  class CollectionItem extends Group {
      get index() {
          const { groups } = this.parent;
          return findIndex(groups, (group) => group === this);
      }
      get formId() {
          return `${this.parent.formId}${this.index}`;
      }
  }
  async function onGroupChanged(...args) {
      const [group] = args;
      if (group.valid) {
          let value = this._d.value;
          if (!value) {
              value = this._d.value = [];
          }
          readonlyDef(value, group.index, () => group.value);
      }
      await onCollectionChanged.apply(this, args);
  }
  async function onCollectionChanged(...args) {
      this._d.pending = true;
      if (this.options.silent) {
          await this.validate({ cascade: false });
      }
      this.emit('changed', this, ...args);
  }
  class Collection extends Element {
      constructor(schema, parent) {
          super(schema, parent);
          const accepted = Collection.accept(schema);
          if (!accepted.valid) {
              throw new Error(logMessage(accepted.reason, accepted.infos));
          }
          readonlyDumpProp(this, 'formType', FORM_TYPE$2);
          readonlyDumpProp(this, 'type', 'set');
          this._d.validation = new Validation(normalizeRules(schema.rules, this.type));
          this._d.value = null;
          this.groups = null;
          if (schema.rules) {
              schema.group.fields = cascadeRules(schema.rules, schema.group.fields);
          }
      }
      static accept(schema) {
          const { accepted, sv } = getSchemaAcceptance(schema, FORM_TYPE$2);
          if (!accepted) {
              const { formId, group } = schema;
              if (!isPlainObject(group)) {
                  invalidateSchemaValidation(sv, '`group` must be an object', { formId });
              }
              if (sv.valid) {
                  acceptSchema(schema, FORM_TYPE$2);
              }
          }
          return sv;
      }
      static create(schema, parent) {
          return new Collection(schema, parent);
      }
      get pending() {
          return this._d.pending;
      }
      get value() {
          return this._d.value;
      }
      async setValue(value, { from = 0, autoAdd = true } = {}) {
          if (!Array.isArray(value)) {
              throw new Error(logMessage('Invalid value, Group value must be an object'));
          }
          const groups = this.groups ? this.groups.slice(from) : [];
          await Promise.all(value.slice(0, autoAdd ? value.length : groups.length).map(async (val, index) => {
              const group = groups[index];
              if (!group) {
                  await this.addGroup().setValue(val);
              }
              else {
                  await group.setValue(val);
              }
          }));
          return this.value;
      }
      shake({ cascade = true } = {}) {
          super.shake();
          if (cascade && this.groups) {
              this.groups.forEach(group => group.shake());
          }
      }
      isValid() {
          return this.validation.valid && (!this.groups || !this.groups.some(g => !g.valid));
      }
      reset() {
          this.cleanUp();
          if (this.groups) {
              this.groups.forEach((group) => group.reset());
          }
          this.validation.reset();
      }
      async clear() {
          this.cleanUp();
          if (this.groups) {
              await Promise.all(this.groups.map(async (group) => await group.clear()));
          }
      }
      addGroup() {
          if (!this.groups) {
              this.groups = [];
          }
          const groupItem = new CollectionItem(this._d.schema.group, this);
          this.groups.push(groupItem);
          groupItem.on('changed:formy', (...args) => onGroupChanged.apply(this, args), { noOff: true });
          return groupItem;
      }
      removeGroup(itemOrIndex) {
          const index = itemOrIndex instanceof CollectionItem ? itemOrIndex.index : itemOrIndex;
          const group = this.groups && this.groups[index];
          if (group) {
              const value = this._d.value;
              this.groups.splice(index, 1);
              if (value) {
                  value.splice(index, 1);
              }
              onCollectionChanged.call(this, group);
          }
      }
      async validate({ cascade = true } = {}) {
          this.emit('validate', this);
          if (cascade && this.groups) {
              await Promise.all(this.groups.map(async (group) => await group.validate()));
          }
          await this.validation.validate(this.value, {}, this.props, this);
          if (!this.valid) {
              this._d.value = null;
          }
          this._d.pending = false;
          this.emit('validated', this);
      }
  }
  Collection.FORM_TYPE = FORM_TYPE$2;

  const defaultOptions = {
      alias: 'forms'
  };
  class Formily {
      constructor(options = {}, $root) {
          this.options = merge({}, defaultOptions, options);
          this.$root = $root;
      }
      addForm(schema) {
          const { options } = this;
          const { rules } = schema;
          schema.rules = merge([], options.rules, rules);
          const form = new Form(schema);
          this.$root.$set(this.$root[options.alias], form.formId, form);
          return form;
      }
      removeForm(formId) {
          delete this.$root[this.options.alias][formId];
      }
      getForm(formId) {
          return this.$root[this.options.alias][formId];
      }
  }

  // defineSchema is a utility that is primarily used for type inference
  // when declaring form schemas. Type inference is provided in the component
  // options (provided as the argument). The returned value has artificial types
  // for TSX / manual render function / IDE support.
  function defineSchema(schema) {
      return schema;
  }

  const _configs = new WeakMap();
  function createFormily() {
      const config = {
          plugs: {},
          elements: []
      };
      const elements = config.elements;
      const proto = Objeto.prototype;
      def(proto, '_config', {
          get() {
              return _configs.get(vueFormily);
          }
      });
      def(proto, 'plugs', {
          get() {
              return this._config.plugs;
          }
      });
      const vueFormily = {
          install(app, options = {}) {
              // initialize default form elements
              [Field, Collection, Group].forEach(F => this.register(F, options));
              app.mixin({
                  beforeCreate() {
                      this.$formily = new Formily(options, this.$root);
                  },
                  data() {
                      const alias = this.$formily.options.alias;
                      let forms = this.$root[alias];
                      if (!forms) {
                          forms = this.$root[alias] = {};
                      }
                      return {
                          [alias]: forms
                      };
                  }
              });
          },
          plug(plugin, ...args) {
              plugin.install(config, ...args);
          },
          register(F, ...args) {
              if (!elements.includes(F)) {
                  elements.unshift(F);
              }
              F.register(...args);
          }
      };
      _configs.set(vueFormily, config);
      return vueFormily;
  }
  const VueFormily = {
      createFormily
  };

  exports.Collection = Collection;
  exports.Evento = Evento;
  exports.Field = Field;
  exports.Form = Form;
  exports.Formily = Formily;
  exports.Group = Group;
  exports.Objeto = Objeto;
  exports.Rule = Rule;
  exports.Validation = Validation;
  exports.VueFormily = VueFormily;
  exports.createFormily = createFormily;
  exports.default = VueFormily;
  exports.defineSchema = defineSchema;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
