import { dumpProp } from '../utils';

function emit(name, ...args) {
  const event = this.events[name];

  if (event) {
    const handlers = event.handlers;
    handlers.forEach(h => h.call(this, ...args));
  }
}

function off(name, handler) {
  const event = this.events[name];

  if (event) {
    const {
      handlers,
      options = {}
    } = event;

    if (!options.noOff) {
      if (handler) {
        handlers.splice(handlers.indexOf(handler), 1);
      } else {
        delete this.events[name];
      }
    }
  }
}

function handleEventHandlers(eventName, fn) {
  Object.keys(this.events).forEach(name => {
    const [n] = parseName(name);

    if (n === eventName) {
      fn(name);
    }
  });
}

function parseName(name) {
  return name.split(':');
}

export default class Evento {
  constructor() {
    dumpProp(this, 'events', {});
  }

  emit(name, ...args) {
    const [eventName, namespace] = parseName(name);

    if (namespace) {
      emit.call(this, name, ...args);
    } else {
      handleEventHandlers.call(this, eventName, n => emit.call(this, n, ...args));
    }

    return this;
  }

  once(name, handler) {
    this.on(name, (...args) => {
      this.off(name, handler);
      handler.call(this, ...args);
    });
    return this;
  }

  on(name, handler, options) {
    const event = this.events[name] || {
      handlers: []
    };
    const handlers = event.handlers;

    if (!handlers.includes(handler)) {
      handlers.push(handler);
    }

    if (options) {
      event.options = options;
    }

    this.events[name] = event;
    return this;
  }

  off(name, handler) {
    const [eventName, namespace] = parseName(name);

    if (namespace) {
      off.call(this, name, handler);
    } else {
      handleEventHandlers.call(this, eventName, n => off.call(this, n, handler));
    }

    return this;
  }

}