import { genHtmlName, getProp, genProps } from '../../helpers';
import { dumpProp, readonlyDumpProp } from '../../utils';
import Objeto from '../Objeto';
import { merge } from '@vue-formily/util';

function genElementAncestors(elem) {
  const path = [];
  let parent = elem.parent;

  while (parent) {
    path.unshift(parent);
    parent = parent.parent;
  }

  return path.length ? path : null;
}

let _options = {
  silent: true
};
export default class Element extends Objeto {
  constructor(schema, parent) {
    super();
    this.props = {};
    this.data = new WeakMap();
    this.shaked = false;
    this.parent = parent || null;
    const data = this._d;
    readonlyDumpProp(data, 'schema', schema);
    const {
      model,
      props = {},
      on = {},
      options
    } = schema;
    readonlyDumpProp(this, 'model', model || this.formId);
    dumpProp(data, 'ancestors', genElementAncestors(this));
    dumpProp(data, 'options', merge({}, _options, options));
    this.addProps(props);
    Object.keys(on).map(name => this.on(name, on[name]));
  }

  static register(options) {
    _options = merge(_options, options);
  }

  get options() {
    return this._d.options;
  }

  get validation() {
    return this._d.validation;
  }

  get error() {
    if (!this.shaked || this.valid) {
      return null;
    }

    return this.validation.errors ? this.validation.errors[0] : null;
  }

  getProps(path, options) {
    return getProp(this, path, options);
  }

  addProps(props, ...args) {
    genProps.call(this, this.props, props, ...args);
  }

  get formId() {
    return this._d.schema.formId || '' + Date.now();
  }

  get htmlName() {
    return this.getHtmlName();
  }

  get valid() {
    return this.isValid();
  }

  getHtmlName() {
    return genHtmlName(this, this._d.ancestors);
  }

  shake() {
    this.shaked = true;
  }

  cleanUp() {
    this.shaked = false;
  }

}