import { isFunction, isPlainObject, isString } from '@vue-formily/util';
import { dumpProp, readonlyDumpProp } from '../../utils';
import Objeto from '../Objeto';
export default class Rule extends Objeto {
  constructor(rule) {
    super();
    readonlyDumpProp(this, 'name', rule.name || Date.now());
    dumpProp(this, 'message', null);
    const data = this._d;
    data.error = null;
    data.valid = true;

    if (isFunction(rule)) {
      this.validator = rule;
    } else if (isPlainObject(rule)) {
      this.validator = rule.validator;
      this.setMessage(rule.message);
    }
  }

  get valid() {
    return this._d.valid;
  }

  get error() {
    return this._d.error;
  }

  setMessage(message = null) {
    this.message = message;
  }

  reset() {
    this._d.error = null;
    this._d.valid = true;
  }

  async validate(value, props = {}, ...args) {
    const {
      _d: data,
      plugs = {}
    } = this;
    const translater = plugs.i18n;
    let error = null;
    let valid = true;
    let result = true;

    if (isFunction(this.validator)) {
      result = await this.validator(value, props, ...args);
    }

    this.emit('validate', this);

    if (result === false || isString(result)) {
      error = result || this.message;
      valid = false;
    }

    data.error = translater && isString(error) ? translater.translate(error, ...args) : error;
    data.valid = valid;
    this.emit('validated', this);
    return this;
  }

}