import { findIndex, isNumber } from '@vue-formily/util';
import Rule from './Rule';
import Objeto from '../Objeto';
export default class Validation extends Objeto {
  constructor(rules) {
    super();
    this.rules = [];

    if (rules) {
      this.addRules(rules);
    }
  }

  get valid() {
    return !this.rules.some(rule => !rule.valid);
  }

  get errors() {
    if (this.valid) {
      return null;
    }

    const errors = this.rules.map(rule => rule.error).filter(error => error);
    return errors.length ? errors : null;
  }

  addRules(rulesOrSchemas, {
    from
  } = {}) {
    from = isNumber(from) ? from++ : -Infinity;
    return rulesOrSchemas.map(schema => {
      return this.addRule(schema, {
        from: from++
      });
    });
  }

  removeRules(removes) {
    return removes.map(remove => this.removeRule(remove));
  }

  addRule(ruleOrSchema, {
    from
  } = {}) {
    const rule = new Rule(ruleOrSchema);
    const currentRule = this[rule.name];

    if (currentRule) {
      this.removeRule(currentRule);
    }

    const length = this.rules.length;
    this.rules.splice(isNumber(from) && from >= 0 && from <= length ? from : length, 0, rule);
    this[rule.name] = rule;
    return rule;
  }

  removeRule(remove) {
    const index = findIndex(this.rules, ({
      name
    }) => {
      const n = remove instanceof Rule ? remove.name : remove;
      return name === n;
    });
    const [removed] = index !== -1 ? this.rules.splice(index, 1) : [];
    delete this[removed.name];
    return removed;
  }

  reset() {
    this.rules.forEach(rule => rule.reset());
  }

  async validate(value, options = {}, ...args) {
    const {
      excluded,
      get
    } = options;
    this.emit('validate', this);

    if (this.rules) {
      let rules = get ? this.rules.filter(({
        name
      }) => get.includes(name)) : this.rules;
      rules = excluded ? rules.filter(({
        name
      }) => !excluded.includes(name)) : rules;
      await Promise.all(rules.map(async rule => await rule.validate(value, ...args)));
    }

    this.emit('validated', this);
    return this;
  }

}